@@ -1,29 +1,32 @@
 /* ltdl.h -- generic dlopen functions
-   Copyright (C) 1998-2000, 2004 Free Software Foundation, Inc.
-   Originally by Thomas Tanner <tanner@ffii.org>
+
+   Copyright (C) 1998-2000, 2004, 2005,
+                 2007, 2008 Free Software Foundation, Inc.
+   Written by Thomas Tanner, 1998
 
    NOTE: The canonical source of this file is maintained with the
    GNU Libtool package.  Report bugs to bug-libtool@gnu.org.
 
-This library is free software; you can redistribute it and/or
+GNU Libltdl is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 As a special exception to the GNU Lesser General Public License,
 if you distribute this file as part of a program or library that
-is built using GNU libtool, you may include it under the same
-distribution terms that you use for the rest of that program.
+is built using GNU Libtool, you may include this file under the
+same distribution terms that you use for the rest of that program.
 
-This library is distributed in the hope that it will be useful,
+GNU Libltdl is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, write to the Free
-Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307  USA
+License along with GNU Libltdl; see the file COPYING.LIB.  If not, a
+copy can be downloaded from  http://www.gnu.org/licenses/lgpl.html,
+or obtained by writing to the Free Software Foundation, Inc.,
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 /* Only include this header file once. */
@@ -40,12 +43,11 @@ LT_BEGIN_C_DECLS
 /* LT_STRLEN can be used safely on NULL pointers.  */
 #define LT_STRLEN(s)	(((s) && (s)[0]) ? strlen (s) : 0)
 
-
 
 /* --- DYNAMIC MODULE LOADING API --- */
 
 
-typedef	void * lt_dlhandle;	/* A loaded module.  */
+typedef	struct lt__handle *lt_dlhandle;	/* A loaded module.  */
 
 /* Initialisation and finalisation functions for libltdl. */
 LT_SCOPE int	    lt_dlinit		(void);
@@ -62,20 +64,25 @@ LT_SCOPE int	    lt_dlforeachfile	 (
 			int (*func) (const char *filename, void *data),
 			void *data);
 
+/* User module loading advisors.  */
+LT_SCOPE int	    lt_dladvise_init	 (lt_dladvise *advise);
+LT_SCOPE int	    lt_dladvise_destroy  (lt_dladvise *advise);
+LT_SCOPE int	    lt_dladvise_ext	 (lt_dladvise *advise);
+LT_SCOPE int	    lt_dladvise_resident (lt_dladvise *advise);
+LT_SCOPE int	    lt_dladvise_local	 (lt_dladvise *advise);
+LT_SCOPE int	    lt_dladvise_global   (lt_dladvise *advise);
+
 /* Portable libltdl versions of the system dlopen() API. */
 LT_SCOPE lt_dlhandle lt_dlopen		(const char *filename);
 LT_SCOPE lt_dlhandle lt_dlopenext	(const char *filename);
+LT_SCOPE lt_dlhandle lt_dlopenadvise	(const char *filename,
+					 lt_dladvise advise);
 LT_SCOPE void *	    lt_dlsym		(lt_dlhandle handle, const char *name);
 LT_SCOPE const char *lt_dlerror		(void);
 LT_SCOPE int	    lt_dlclose		(lt_dlhandle handle);
 
-/* Module residency management. */
-LT_SCOPE int	    lt_dlmakeresident	(lt_dlhandle handle);
-LT_SCOPE int	    lt_dlisresident	(lt_dlhandle handle);
 
 
-
-
 /* --- PRELOADED MODULE SUPPORT --- */
 
 
@@ -84,11 +91,6 @@ LT_SCOPE int	    lt_dlisresident	(lt_dlhandle handle);
 typedef struct {
   const char *name;
   void       *address;
-} lt_dlsymbol;
-
-typedef struct {
-  const char    *originator;
-  const lt_dlsymbol symbols[];
 } lt_dlsymlist;
 
 typedef int lt_dlpreload_callback_func (lt_dlhandle handle);
@@ -100,8 +102,8 @@ LT_SCOPE int	lt_dlpreload_open    (const char *originator,
 
 #define lt_preloaded_symbols	lt__PROGRAM__LTX_preloaded_symbols
 #define LTDL_SET_PRELOADED_SYMBOLS() 			LT_STMT_START{	\
-	extern const lt_dlsymlist lt_preloaded_symbols;			\
-	lt_dlpreload_default(&lt_preloaded_symbols);			\
+	extern const lt_dlsymlist lt_preloaded_symbols[];		\
+	lt_dlpreload_default(lt_preloaded_symbols);			\
 							}LT_STMT_END
 
 
@@ -111,15 +113,17 @@ LT_SCOPE int	lt_dlpreload_open    (const char *originator,
 
 
 /* Associating user data with loaded modules. */
-typedef void * lt_dlcaller_id;
+typedef void * lt_dlinterface_id;
 typedef int lt_dlhandle_interface (lt_dlhandle handle, const char *id_string);
 
-LT_SCOPE lt_dlcaller_id	lt_dlcaller_register  (const char *id_string,
-					       lt_dlhandle_interface *iface);
-LT_SCOPE void *		lt_dlcaller_set_data  (lt_dlcaller_id key,
-					       lt_dlhandle handle, void *data);
-LT_SCOPE void *		lt_dlcaller_get_data  (lt_dlcaller_id key,
-					       lt_dlhandle handle);
+LT_SCOPE lt_dlinterface_id lt_dlinterface_register (const char *id_string,
+					  lt_dlhandle_interface *iface);
+LT_SCOPE void	lt_dlinterface_free (lt_dlinterface_id key);
+LT_SCOPE void *	lt_dlcaller_set_data  (lt_dlinterface_id key,
+					  lt_dlhandle handle, void *data);
+LT_SCOPE void *	lt_dlcaller_get_data  (lt_dlinterface_id key,
+					  lt_dlhandle handle);
+
 
 /* Read only information pertaining to a loaded module. */
 typedef	struct {
@@ -127,52 +131,30 @@ typedef	struct {
   char *	name;		/* module name */
   int		ref_count;	/* number of times lt_dlopened minus
 				   number of times lt_dlclosed. */
+  unsigned int	is_resident:1;	/* module can't be unloaded. */
+  unsigned int	is_symglobal:1;	/* module symbols can satisfy
+				   subsequently loaded modules.  */
+  unsigned int	is_symlocal:1;	/* module symbols are only available
+				   locally. */
 } lt_dlinfo;
 
 LT_SCOPE const lt_dlinfo *lt_dlgetinfo	    (lt_dlhandle handle);
-LT_SCOPE lt_dlhandle	lt_dlhandle_first   (lt_dlcaller_id key);
-LT_SCOPE lt_dlhandle	lt_dlhandle_next    (lt_dlhandle place);
-LT_SCOPE lt_dlhandle	lt_dlhandle_find    (const char *module_name);
-LT_SCOPE int		lt_dlforeach	    (
+
+LT_SCOPE lt_dlhandle	lt_dlhandle_iterate (lt_dlinterface_id iface,
+					     lt_dlhandle place);
+LT_SCOPE lt_dlhandle	lt_dlhandle_fetch   (lt_dlinterface_id iface,
+					     const char *module_name);
+LT_SCOPE int		lt_dlhandle_map	    (lt_dlinterface_id iface,
 				int (*func) (lt_dlhandle handle, void *data),
 				void *data);
 
 
-
-/* --- BINARY COMPATIBILITY WITH OLD LIBLTDL --- */
 
-typedef void	lt_dlmutex_lock		(void);
-typedef void	lt_dlmutex_unlock	(void);
-typedef void	lt_dlmutex_seterror	(const char *errmsg);
-typedef const char *lt_dlmutex_geterror	(void);
+/* Deprecated module residency management API. */
+LT_SCOPE int	    lt_dlmakeresident	(lt_dlhandle handle);
+LT_SCOPE int	    lt_dlisresident	(lt_dlhandle handle);
 
-LT_SCOPE void * (*lt_dlmalloc)	(size_t size);
-LT_SCOPE void * (*lt_dlrealloc)	(void *ptr, size_t size);
-LT_SCOPE void	(*lt_dlfree)	(void *ptr);
-
-
-LT_SCOPE int	lt_dlmutex_register	(lt_dlmutex_lock *lock,
-					 lt_dlmutex_unlock *unlock,
-					 lt_dlmutex_seterror *seterror,
-					 lt_dlmutex_geterror *geterror);
-
-# define lt_ptr			void *
-
-
-
-/* --- SOURCE COMPATIBILITY WITH ANCIENT LIBLTDL --- */
-
-
-#if defined(LT_NON_POSIX_NAMESPACE)
-#  define lt_ptr_t		void *
-#  define lt_module_t		lt_module
-#  define lt_module_open_t	lt_module_open
-#  define lt_module_close_t	lt_module_close
-#  define lt_find_sym_t		lt_find_sym
-#  define lt_dlloader_exit_t	lt_dlloader_exit
-#  define lt_dlloader_t		lt_dlloader
-#  define lt_dlloader_data_t	lt_user_data
-#endif
+#define lt_ptr void *
 
 LT_END_C_DECLS
 
